@@ -2,7 +2,7 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore udev pcent iuse itotal iused ipcent
+// spell-checker:ignore udev pcent iuse itotal iused ipcent binfmt
 #![allow(
     clippy::similar_names,
     clippy::cast_possible_truncation,
@@ -648,6 +648,53 @@ fn test_block_size_with_suffix() {
     assert_eq!(get_header("1GB"), "1GB-blocks");
 }
 
+#[test]
+fn test_df_binary_block_size() {
+    fn get_header(block_size: &str) -> String {
+        let output = new_ucmd!()
+            .args(&["-B", block_size, "--output=size"])
+            .succeeds()
+            .stdout_str_lossy();
+        output.lines().next().unwrap().trim().to_string()
+    }
+
+    let test_cases = [
+        ("0b1", "1"),
+        ("0b10100", "20"),
+        ("0b1000000000", "512"),
+        ("0b10K", "2K"),
+    ];
+
+    for (binary, decimal) in test_cases {
+        let binary_result = get_header(binary);
+        let decimal_result = get_header(decimal);
+        assert_eq!(
+            binary_result, decimal_result,
+            "Binary {binary} should equal decimal {decimal}"
+        );
+    }
+}
+
+#[test]
+fn test_df_binary_env_block_size() {
+    fn get_header(env_var: &str, env_value: &str) -> String {
+        let output = new_ucmd!()
+            .env(env_var, env_value)
+            .args(&["--output=size"])
+            .succeeds()
+            .stdout_str_lossy();
+        output.lines().next().unwrap().trim().to_string()
+    }
+
+    let binary_header = get_header("DF_BLOCK_SIZE", "0b10000000000");
+    let decimal_header = get_header("DF_BLOCK_SIZE", "1024");
+    assert_eq!(binary_header, decimal_header);
+
+    let binary_header = get_header("BLOCK_SIZE", "0b10000000000");
+    let decimal_header = get_header("BLOCK_SIZE", "1024");
+    assert_eq!(binary_header, decimal_header);
+}
+
 #[test]
 fn test_block_size_in_posix_portability_mode() {
     fn get_header(block_size: &str) -> String {
@@ -849,6 +896,32 @@ fn test_invalid_block_size_suffix() {
         .stderr_contains("invalid suffix in --block-size argument '1.2'");
 }
 
+#[test]
+fn test_df_invalid_binary_size() {
+    new_ucmd!()
+        .arg("--block-size=0b123")
+        .fails()
+        .stderr_contains("invalid suffix in --block-size argument '0b123'");
+}
+
+#[test]
+fn test_df_binary_edge_cases() {
+    new_ucmd!()
+        .arg("-B0b")
+        .fails()
+        .stderr_contains("invalid --block-size argument '0b'");
+
+    new_ucmd!()
+        .arg("-B0B")
+        .fails()
+        .stderr_contains("invalid suffix in --block-size argument '0B'");
+
+    new_ucmd!()
+        .arg("--block-size=0b1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111")
+        .fails()
+        .stderr_contains("too large");
+}
+
 #[test]
 fn test_output_selects_columns() {
     let output = new_ucmd!()
@@ -973,3 +1046,48 @@ fn test_nonexistent_file() {
         .stderr_is("df: does-not-exist: No such file or directory\n")
         .stdout_is("File\n.\n");
 }
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_df_all_shows_binfmt_misc() {
+    // Check if binfmt_misc is mounted
+    let is_mounted = std::fs::read_to_string("/proc/self/mountinfo")
+        .map(|content| content.lines().any(|line| line.contains("binfmt_misc")))
+        .unwrap_or(false);
+
+    if is_mounted {
+        let output = new_ucmd!()
+            .args(&["--all", "--output=fstype,target"])
+            .succeeds()
+            .stdout_str_lossy();
+
+        assert!(
+            output.contains("binfmt_misc"),
+            "Expected binfmt_misc filesystem to appear in df --all output when it's mounted"
+        );
+    }
+    // If binfmt_misc is not mounted, skip the test silently
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_df_hides_binfmt_misc_by_default() {
+    // Check if binfmt_misc is mounted
+    let is_mounted = std::fs::read_to_string("/proc/self/mountinfo")
+        .map(|content| content.lines().any(|line| line.contains("binfmt_misc")))
+        .unwrap_or(false);
+
+    if is_mounted {
+        let output = new_ucmd!()
+            .args(&["--output=fstype,target"])
+            .succeeds()
+            .stdout_str_lossy();
+
+        // binfmt_misc should NOT appear in the output without --all
+        assert!(
+            !output.contains("binfmt_misc"),
+            "Expected binfmt_misc filesystem to be hidden in df output without --all"
+        );
+    }
+    // If binfmt_misc is not mounted, skip the test silently
+}
